/*
 * valout.c
 * http://www.mikrocontroller.net/topic/121526#new
 * 20110822 de0508, Uwe
 */
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "valout.h"

#ifdef VALOUT_32
# define	TABLE_READ(ADDR)	pgm_read_dword (ADDR)
# define	table_read(x,y)		pgm_read_dword((x)+y)

#else
# define	TABLE_READ(ADDR)	pgm_read_word (ADDR)
# define	table_read(x,y)		pgm_read_word((x)+y)
#endif


#define APPEND_ZERO				// append 0 to output string

#ifndef VALOUT_DECIMAL_POINT
# define VALOUT_DECIMAL_POINT	','
#endif  // VALOUT_DECIMAL_POINT

// maximum number length
// 2.40823 = log 256 / log 10
static uint8_t buff[(int) 2.40823*sizeof(valout_data_t)+1 + sizeof ("-.")];		

static const valout_data_t TEST[] PROGMEM =
{
             10L,
            100L,
           1000L,
          10000L,
#ifdef VALOUT_32
         100000L,
        1000000L,
       10000000L,
      100000000L,
     1000000000L,
#endif // VALOUT_32
};

#define TABLE_SIZE ( sizeof(TEST) / sizeof(valout_data_t) )

void valout (valout_data_t val, int8_t digits, int8_t point, char * outp)
{
  uint8_t notzero;
  uint8_t *buffp = buff;

  // handle sign
  if( val < 0 )
  {
    val = -val;
    *buffp++ = '-';
  }

  point -= TABLE_SIZE + 1;
  
  notzero = 0;

  for( uint8_t i = TABLE_SIZE; i; i-- )
  {
    uint8_t d;

    // spart uns einen Datenabruf
    //valout_data_t power_10_i = (valout_data_t) table_read(TEST, i-1);
    valout_data_t power_10_i = (valout_data_t) TABLE_READ( &TEST[i-1] );

    for( d = '0';
      val >= power_10_i;
      val -= power_10_i )
    {
      d++;
      notzero = 1;
    }

    if( notzero )
      *buffp++ = d ;

    if( ++point == 0 )			// insert decimal point
    {
      if( notzero == 0 )
	*buffp++ = d;			// zero prior decimal point
      notzero = 1;
      *buffp++ = VALOUT_DECIMAL_POINT;		// decimal point sign
    }
  }

  *buffp++ = val + '0';			// remainder = ones
  *buffp = '\0';

  // length of string
  // digits -= buffp - buff;
  digits -= ((int) buffp) - ((int) buff);

  while( --digits >= 0 )
    *outp++ = ' ';			// insert spaces until min length
  buffp = buff;

#ifdef APPEND_ZERO
  do
    *outp++ = *buffp;
  while( *buffp++ );			// copy with '\0'
#else
  while( *buffp )
    *outp++ = *buffp++;			// copy without '\0'
#endif
}
